{=========================================================================
  RkTrkBar Unit

  The TRkTrackBar component is a slider control that mimics the behavior
  of the Windows 95 TrackBar control. This control works with mouse -and-
  keyboard input.

  Developing Custom Delphi 3 Components
  Copyright  1995-1997 by Ray Konopka
=========================================================================}

{$I DCDC.INC}

unit RkTrkBar;

interface

uses
  Messages, Windows, Classes, Graphics, Controls, Menus,
  ExtCtrls, RkCommon;

type
  TTrackOrientation = ( toHorizontal, toVertical );
  TThumbSize = ( tsSmall, tsMedium, tsLarge );
  TThumbStyle = ( tsBox, tsPointer );
  TTickStyle = ( tkStandard, tkOwnerDraw );

  TRkTrackBar = class;                          // Forward class reference

  TDrawTickEvent = procedure ( TrackBar : TRkTrackBar; Canvas : TCanvas;
                               Location : TPoint;
                               Index : Integer ) of object;

  TPositionChangingEvent = procedure( Sender : TObject; NewPos : Integer;
                                    var AllowChange : Boolean ) of object;

  TRkTrackBar = class( TCustomControl )
  private
    FAboutInfo : TRkAboutInfo;
    FBorderWidth : Integer;
    FMax : Integer;
    FMin : Integer;
    FOrientation : TTrackOrientation;
    FPageSize : Word;
    FPosition : Integer;
    FTickStyle : TTickStyle;
    FShowTicks : Boolean;
    FSliding : Boolean;

    // Internal storage for Thumb properties
    FThumbHeight : Integer;
    FThumbRct : TRect;
    FLastThumbRct : TRect;
    FThumbSize : TThumbSize;
    FThumbStyle : TThumbStyle;
    FThumbWidth : Integer;
    FHalfWidth : Integer;

    // Internal storage for Track properties
    FTrackColor : TColor;
    FTrackRct : TRect;
    FTrackWidth : Word;

    FDitherBmp : TBitmap;
    FThumbBmp : TBitmap;
    FMaskBmp : TBitmap;
    FBackgroundBmp : TBitmap;
    FThumbCursor : HCursor;

    // Internal storage for custom events
    FOnChange : TNotifyEvent;
    FOnChanging : TPositionChangingEvent;
    FOnDrawTick : TDrawTickEvent;

    procedure SetMax( Value : Integer );
    procedure SetMin( Value : Integer );
    procedure SetOrientation( Value : TTrackOrientation );
    procedure SetPosition( Value : Integer );
    procedure SetShowTicks( Value : Boolean );
    procedure SetThumbSize( Value : TThumbSize );
    procedure SetThumbStyle( Value : TThumbStyle );
    procedure SetTickStyle( Value : TTickStyle );
    procedure SetTrackWidth( Value : Word );
    procedure SetTrackColor( Value : TColor );
    procedure LoadThumbBitmaps;
    procedure UpdateDitherBitmap;
    procedure WMGetDlgCode(var Msg: TWMGetDlgCode); message wm_GetDlgCode;
    procedure WMSetCursor( var Msg : TWMSetCursor ); message wm_SetCursor;
    procedure WMSize( var Msg : TWMSize); message wm_Size;
    procedure CMEnabledChanged( var Msg : TMessage );
      message cm_EnabledChanged;
    function CursorPosition : TPoint;
    procedure CMDesignHitTest( var Msg : TCMDesignHitTest );
      message cm_DesignHitTest;
  protected
    procedure Loaded; override;
    procedure DrawTrack; virtual;
    procedure DrawTicks; virtual;
    procedure DrawThumb; virtual;
    procedure Paint; override;

    procedure Change; dynamic;
    function CanChange( NewPos : Integer ) : Boolean; dynamic;

    procedure DrawTick( Canvas : TCanvas; Location : TPoint;
                        Index : Integer ); dynamic;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure KeyDown( var Key : Word; Shift : TShiftState ); override;
    procedure MouseDown( Button : TMouseButton; Shift : TShiftState;
                         X, Y : Integer ); override;
    procedure MouseMove( Shift : TShiftState; X, Y : Integer ); override;
    procedure MouseUp( Button : TMouseButton; Shift : TShiftState;
                       X, Y : Integer ); override;
  public
    constructor Create( AOwner : TComponent ); override;
    destructor Destroy; override;
  published
    property About : TRkAboutInfo
      read FAboutInfo
      write FAboutInfo
      stored False;

    property Max : Integer
      read FMax
      write SetMax
      default 10;

    property Min : Integer
      read FMin
      write SetMin
      default 0;

    property Orientation : TTrackOrientation
      read FOrientation
      write SetOrientation
      default toHorizontal;

    property PageSize : Word
      read FPageSize
      write FPageSize
      default 1;

    property Position : Integer
      read FPosition
      write SetPosition;

    property ShowTicks : Boolean
      read FShowTicks
      write SetShowTicks
      default True;

    property ThumbSize : TThumbSize
      read FThumbSize
      write SetThumbSize
      default tsMedium;

    property ThumbStyle : TThumbStyle
      read FThumbStyle
      write SetThumbStyle
      default tsPointer;

    property TickStyle : TTickStyle
      read FTickStyle
      write SetTickStyle
      default tkStandard;

    property TrackColor : TColor
      read FTrackColor
      write SetTrackColor
      default clWhite;

    property TrackWidth : Word
      read FTrackWidth
      write SetTrackWidth
      default 8;

    property OnChange : TNotifyEvent
      read FOnChange
      write FOnChange;

    property OnChanging : TPositionChangingEvent
      read FOnChanging
      write FOnChanging;

    property OnDrawTick : TDrawTickEvent
      read FOnDrawTick
      write FOnDrawTick;

    { Inherited Properties & Events }
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property HelpContext;
    property Hint;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop default True;
    property Visible;

    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;


implementation

{$R RKTRKBAR.RES}                            // Link in bitmaps for thumbs

uses
  SysUtils, Forms;

{=========================}
{== TRkTrackBar Methods ==}
{=========================}

constructor TRkTrackBar.Create( AOwner : TComponent );
begin
  inherited Create( AOwner );
  Width := 200;
  Height := 50;
  FTrackWidth := 8;
  FOrientation := toHorizontal;
  FTrackColor := clWhite;
  FMin := 0;
  FMax := 10;
  FPosition := 0;
  FBorderWidth := 4;
  FPageSize := 1;
  TabStop := True;
  FShowTicks := True;
  FSliding := False;

  FThumbBmp := TBitmap.Create;           // Create internal bitmap objects
  FMaskBmp := TBitmap.Create;
  FBackgroundBmp := TBitmap.Create;
  FThumbStyle := tsPointer;
  FThumbSize := tsMedium;
  FDitherBmp := TBitmap.Create;
  FDitherBmp.Width := 8;
  FDitherBmp.Height := 8;
  UpdateDitherBitmap;
  LoadThumbBitmaps;

  FThumbCursor := LoadCursor( HInstance, 'RKTRKBAR_THUMBCURSOR' );
end;


destructor TRkTrackBar.Destroy;
begin
  FDitherBmp.Free;         // Be sure to free internally allocated objects
  FThumbBmp.Free;
  FMaskBmp.Free;
  FBackgroundBmp.Free;
  DestroyCursor( FThumbCursor );              // Release GDI cursor object
  inherited Destroy;
end;


procedure TRkTrackBar.Loaded;
begin
  inherited Loaded;
  // Need to reinitialize the FThumbRct so that the test between FThumbRct
  // and FLastThumbRct fails. This ensures that the correct background
  // image is captured when the component is loaded.
  FThumbRct := Rect( 0, 0, 0, 0 );
end;


procedure TRkTrackBar.SetMax( Value : Integer );
begin
  if FMax <> Value then
  begin
    FMax := Value;
    if FMax <= FMin then
      FMax := FMin + 1;                             // Max cannot be < Min

    if FPosition > FMax then           // If new max < current Position...
      Position := FMax;                    // Update the Position property
    Invalidate;
  end;
end;


procedure TRkTrackBar.SetMin( Value : Integer );
begin
  if FMin <> Value then
  begin
    FMin := Value;
    if FMin >= FMax then
      FMin := FMax - 1;                             // Min cannot be > Max

    if FPosition < FMin then           // If new min > current Position...
      Position := FMin;                    // Update the Position property
    Invalidate;
  end;
end;


procedure TRkTrackBar.SetOrientation( Value : TTrackOrientation );
begin
  if FOrientation <> Value then
  begin
    FOrientation := Value;
    LoadThumbBitmaps;            // Get new bitmaps if Orientation changes
    Invalidate;
  end;
end;


{=========================================================================
  TRkTrackBar.CanChange

  This is the event dispatch method supporting the OnChanging event.
  Notice that this method is a function, rather than the common procedure
  variety. As a function, the Result variable is assigned a value before
  calling the user defined event handler.
=========================================================================}

function TRkTrackBar.CanChange( NewPos : Integer ) : Boolean;
begin
  Result := True;
  if Assigned( FOnChanging ) then
    FOnChanging( Self, NewPos, Result );
end;


procedure TRkTrackBar.SetPosition( Value : Integer );
begin
  if FPosition <> Value then
  begin
    // Range Checking
    if Value < FMin then
      Value := FMin
    else if Value > FMax then
      Value := FMax;

    if CanChange( Value ) then
    begin
      FPosition := Value;

      // Erase old thumb image by drawing background bitmap
      Canvas.Draw( FThumbRct.Left, FThumbRct.Top, FBackgroundBmp );

      DrawThumb;                             // Draw thumb at new location
      Change;                                      // Trigger Change event

      // Force the Object Inspector to update Position property display
      if ( csDesigning in ComponentState ) and
         ( GetParentForm( Self ) <> nil ) then
        GetParentForm( Self ).Designer.Modified;
    end;
  end;
end; {= TRkTrackBar.SetPosition =}


procedure TRkTrackBar.SetShowTicks( Value : Boolean );
begin
  if Value <> FShowTicks then
  begin
    FShowTicks := Value;
    Invalidate;
  end;
end;


procedure TRkTrackBar.SetThumbSize( Value : TThumbSize );
begin
  if Value <> FThumbSize then
  begin
    FThumbSize := Value;
    LoadThumbBitmaps;              // Reload bitmaps if thumb size changes
    Invalidate;
  end;
end;


procedure TRkTrackBar.SetThumbStyle( Value : TThumbStyle );
begin
  if Value <> FThumbStyle then
  begin
    FThumbStyle := Value;
    LoadThumbBitmaps;             // Reload bitmaps if thumb style changes
    Invalidate;
  end;
end;


procedure TRkTrackBar.SetTickStyle( Value : TTickStyle );
begin
  if Value <> FTickStyle then
  begin
    FTickStyle := Value;
    Invalidate;
  end;
end;


procedure TRkTrackBar.SetTrackColor( Value : TColor );
begin
  if Value <> FTrackColor then
  begin
    FTrackColor := Value;
    UpdateDitherBitmap;
    Invalidate;
  end;
end;


procedure TRkTrackBar.SetTrackWidth( Value : Word );
begin
  if FTrackWidth <> Value then
  begin
    FTrackWidth := Value;
    if FTrackWidth < 6 then
      FTrackWidth := 6;
    Invalidate;
  end;
end;


// Array Constants hold all bitmap resource names for easy access

const
  ThumbBitmapNames : array[ TTrackOrientation, TThumbSize ] of PChar =
    ( ( 'RKTRKBAR_SMHORZTHUMB',
        'RKTRKBAR_MEDHORZTHUMB',
        'RKTRKBAR_LGHORZTHUMB' ),
      ( 'RKTRKBAR_SMVERTTHUMB',
        'RKTRKBAR_MEDVERTTHUMB',
        'RKTRKBAR_LGVERTTHUMB' ) );
  MaskBitmapNames : array[ TTrackOrientation, TThumbSize ] of PChar =
    ( ( 'RKTRKBAR_SMHORZTHUMBMASK',
        'RKTRKBAR_MEDHORZTHUMBMASK',
        'RKTRKBAR_LGHORZTHUMBMASK' ),
      ( 'RKTRKBAR_SMVERTTHUMBMASK',
        'RKTRKBAR_MEDVERTTHUMBMASK',
        'RKTRKBAR_LGVERTTHUMBMASK' ) );
  BoxBitmapNames : array[ TTrackOrientation, TThumbSize ] of PChar =
    ( ( 'RKTRKBAR_SMHORZBOX',
        'RKTRKBAR_MEDHORZBOX',
        'RKTRKBAR_LGHORZBOX' ),
      ( 'RKTRKBAR_SMVERTBOX',
        'RKTRKBAR_MEDVERTBOX',
        'RKTRKBAR_LGVERTBOX' ) );


procedure TRkTrackBar.LoadThumbBitmaps;
begin
  if FThumbStyle = tsPointer then
  begin
    FThumbBmp.LoadFromResourceName( HInstance,
                                    ThumbBitmapNames[ FOrientation,
                                                      FThumbSize ]);
    FMaskBmp.LoadFromResourceName( HInstance,
                                   MaskBitmapNames[ FOrientation,
                                                    FThumbSize ] );
  end
  else
  begin
    FThumbBmp.LoadFromResourceName( HInstance,
                                    BoxBitmapNames[ FOrientation,
                                                    FThumbSize ]);
  end;

  if FOrientation = toVertical then
  begin
    FThumbHeight := FThumbBmp.Width;
    FThumbWidth := FThumbBmp.Height;
  end
  else
  begin
    FThumbHeight := FThumbBmp.Height;
    FThumbWidth := FThumbBmp.Width;
  end;
  FHalfWidth := FThumbWidth div 2;
end; {= TRkTrackBar.LoadThumbBitmaps =}


procedure TRkTrackBar.UpdateDitherBitmap;
var
  C : TColor;
  I, J : Integer;
begin
  C := clSilver;
  if ColorToRGB( FTrackColor) = clSilver then
    C := clGray;

  with FDitherBmp.Canvas do
  begin
    Brush.Color := FTrackColor;
    FillRect( Rect( 0, 0, FDitherBmp.Width, FDitherBmp.Height ) );
    for I := 0 to 7 do
      for J := 0 to 7 do
        if ( I + J ) mod 2 <> 0 then
          Pixels[ I, J ] := C;
  end;
end; {= TRkTrackBar.UpdateDitherBitmap =}


procedure TRkTrackBar.DrawTrack;
begin
  // Calculate the size of the track
  if FOrientation = toVertical then
  begin
    FTrackRct.Top := FHalfWidth + FBorderWidth;
    FTrackRct.Bottom := Height - FBorderWidth - FHalfWidth;
    FTrackRct.Left := ( Width - FTrackWidth ) div 2;
    FTrackRct.Right := FTrackRct.Left + FTrackWidth;
  end
  else
  begin
    FTrackRct.Top := ( Height - FTrackWidth ) div 2;
    FTrackRct.Bottom := FTrackRct.Top + FTrackWidth;
    FTrackRct.Left := FHalfWidth + FBorderWidth;
    FTrackRct.Right := Width - FBorderWidth - FHalfWidth;
  end;

  // Draw the Track
  Canvas.Brush.Color := FTrackColor;

  if not Enabled then
    Canvas.Brush.Bitmap := FDitherBmp;

  Canvas.FillRect( FTrackRct );
  DrawCtl3DBorder( Canvas, FTrackRct );              // From RkCommon unit
end; {= TRkTrackBar.DrawTrack =}


procedure TRkTrackBar.DrawTicks;
var
  Delta : Real;
  I, X, Y : Integer;
begin
  Canvas.Pen.Color := clBlack;
  with FTrackRct do
  begin
    if FOrientation = toVertical then
    begin
      // Delta is spacing between tick marks
      Delta := (Height - FThumbWidth - 2 * FBorderWidth) / (FMax - FMin);

      for I := FMin to FMax do
      begin
        Y := Height - FThumbWidth - Trunc(Delta*(I-FMin)) - FBorderWidth;

        if FTickStyle = tkStandard then
        begin
          Canvas.MoveTo( FBorderWidth, Y + FHalfWidth );
          Canvas.LineTo( 10, Y + FHalfWidth );
          if FThumbStyle = tsBox then          // Draw ticks on other side
          begin
            Canvas.MoveTo( Width - 10, Y + FHalfWidth );
            Canvas.LineTo( Width - FBorderWidth, Y + FHalfWidth );
          end;
        end
        else                           // Provide hook to owner draw ticks
          DrawTick( Canvas, Point( 0, Y + FHalfWidth ), I );
      end;
    end
    else
    begin
      Delta := (Width - FThumbWidth - 2 * FBorderWidth) / ( FMax - FMin );

      for I := FMin to FMax do
      begin
        X := Trunc( Delta * ( I - FMin ) ) + FBorderWidth;

        if FTickStyle = tkStandard then
        begin
          Canvas.MoveTo( X + FHalfWidth, FBorderWidth );
          Canvas.LineTo( X + FHalfWidth, 10 );
          if FThumbStyle = tsBox then          // Draw ticks on other side
          begin
            Canvas.MoveTo( X + FHalfWidth, Height - 10 );
            Canvas.LineTo( X + FHalfWidth, Height - FBorderWidth );
          end;
        end
        else                           // Provide hook to owner draw ticks
          DrawTick( Canvas, Point( X + FHalfWidth, 0 ), I );
      end;
    end;
  end;
end; {= TRkTrackBar.DrawTicks =}


procedure TRkTrackBar.DrawThumb;
var
  Offset : Longint;
  WorkBmp : TBitmap;
  WorkRct : TRect;
begin
  FLastThumbRct := FThumbRct;
  // Calculate new location of thumb based on Position
  if FOrientation = toVertical then
  begin
    Offset := ( Longint( Height ) - FThumbWidth - 2 * FBorderWidth ) *
              ( FPosition - FMin ) div ( FMax - FMin );
    FThumbRct.Left := ( Width - FThumbHeight ) div 2;
    FThumbRct.Right := FThumbRct.Left + FThumbHeight;
    FThumbRct.Bottom := Height - Offset - FBorderWidth;
    FThumbRct.Top := FThumbRct.Bottom - FThumbWidth;
  end
  else
  begin
    Offset := ( Longint( Width ) - FThumbWidth - 2 * FBorderWidth ) *
              ( FPosition - FMin ) div ( FMax - FMin );
    FThumbRct.Left := Offset + FBorderWidth;
    FThumbRct.Right := FThumbRct.Left + FThumbWidth;
    FThumbRct.Top := ( Height - FThumbHeight ) div 2;
    FThumbRct.Bottom := FThumbRct.Top + FThumbHeight;
  end;

  // The Delphi IDE calls the Paint method of whichever control is
  // selected.  When a new component is selected, the previous component
  // is repainted.  However, there is no focus change at design-time.
  // As a result, when the track bar gets repainted, it captures the
  // thumb itself as part of the background.  This is because the thumb
  // rectangle has not been changed.

  if not ( csDesigning in ComponentState ) or
     not EqualRect( FThumbRct, FLastThumbRct ) then
  begin
    // Save background image of new thumb location
    FBackgroundBmp.Width := FThumbBmp.Width;
    FBackgroundBmp.Height := FThumbBmp.Height;
    FBackgroundBmp.Canvas.CopyRect( Rect( 0, 0,
                                          FThumbBmp.Width,
                                          FThumbBmp.Height ),
                                    Canvas, FThumbRct );
  end;

  // Draw the thumb by displaying the thumb bitmap

  // WorkBmp is used to combine the Thumb bitmap and the background so
  // that the background of the track appears in the corners of the
  // Thumb image.

  // If ThumbStyle is tsBox, there is no need to mask out the background,
  // so just copy the thumb image to the control canvas.

  WorkBmp := TBitmap.Create;
  try
    // Don't forget to set working bitmap size to that of thumb bitmap
    WorkBmp.Height := FThumbBmp.Height;
    WorkBmp.Width := FThumbBmp.Width;

    // WorkRct specifies the Width and Height of the region we are
    // dealing with
    WorkRct := Rect( 0, 0, FThumbBmp.Width, FThumbBmp.Height );

    if FThumbStyle = tsPointer then
    begin
      // Copy the FBackgroundBmp image to WorkBmp

      WorkBmp.Canvas.CopyMode := cmSrcCopy;
      WorkBmp.Canvas.CopyRect( WorkRct, FBackgroundBmp.Canvas, WorkRct );

      // Combine the FBackgroundBmp and the FMaskBmp images using the
      // cmSrcAnd CopyMode. White pixels in mask have no effect.
      // Background shows thru.

      WorkBmp.Canvas.CopyMode := cmSrcAnd;
      WorkBmp.Canvas.CopyRect( WorkRct, FMaskBmp.Canvas, WorkRct );

      // Copy the Thumb bitmap onto the Working bitmap using the
      // cmSrcPaint mode. Black pixels in Thumb bitmap let background
      // show through.

      WorkBmp.Canvas.CopyMode := cmSrcPaint
    end
    else
      WorkBmp.Canvas.CopyMode := cmSrcCopy;

    WorkBmp.Canvas.CopyRect( WorkRct, FThumbBmp.Canvas, WorkRct );

    if not Enabled then
    begin
      // If control is disabled, dither the thumb as well as the track
      WorkBmp.Canvas.Brush.Bitmap := FDitherBmp;
      WorkBmp.Canvas.FloodFill( WorkRct.Right - 3, WorkRct.Bottom - 3,
                                clSilver, fsSurface );
    end;

    // Copy the working bitmap onto the control's Canvas at thumb position
    Canvas.CopyRect( FThumbRct, WorkBmp.Canvas, WorkRct );
  finally
    WorkBmp.Free;
  end;
end; {= TRkTrackBar.DrawThumb =}


procedure TRkTrackBar.Paint;
begin
  with Canvas do
  begin
    // Indicate focus by drawing dotted box around control
    if Focused then
      DrawFocusRect( ClientRect );
    DrawTrack;
    if FShowTicks then
      DrawTicks;
    DrawThumb;
  end;
end;


procedure TRkTrackBar.Change;
begin
  if Assigned( FOnChange ) then
    FOnChange( Self );
end;


{=========================================================================
  TRkTrackBar.DrawTick

  This method is the event dispatch method for the OnDrawTick event.
  The parameters are:
    Canvas - The Canvas for the TrackBar Control
    Location - Point record indicating X or Y coordinates of tick mark
    Index - Position index of tick mark to be drawn
=========================================================================}

procedure TRkTrackBar.DrawTick( Canvas : TCanvas; Location : TPoint;
                              Index : Integer );
begin
  if Assigned( FOnDrawTick ) then       // Allow user to draw custom ticks
    FOnDrawTick( Self, Canvas, Location, Index );
end;


procedure TRkTrackBar.DoEnter;
begin
  inherited DoEnter;
  // When control gets focus, update display to show focus border
  Refresh;
end;


procedure TRkTrackBar.DoExit;
begin
  inherited DoExit;
  // When control loses focus, update display to remove focus border
  Refresh;
end;


procedure TRkTrackBar.KeyDown( var Key : Word; Shift : TShiftState );
begin
  inherited KeyDown( Key, Shift );

  case Key of
    vk_Prior:                             // PgUp Key - increases Position
      Position := FPosition + FPageSize;

    vk_Next:                              // PgDn Key - decreases Position
      Position := FPosition - FPageSize;

    vk_End:
      if FOrientation = toVertical then   // End is at
        Position := FMin                  // Right for horizontal TrackBar
      else                                // Bottom for vertical TrackBar
        Position := FMax;

    vk_Home:
      if FOrientation = toVertical then    // Home is at
        Position := FMax                   // Left for horizontal TrackBar
      else                                 // Top for vertical TrackBar
        Position := FMin;

    vk_Left:                                          // Decrease Position
      if FPosition > FMin then
        Position := FPosition - 1;

    vk_Up:                                            // Increase Position
      if FPosition < FMax then
        Position := FPosition + 1;

    vk_Right:                                         // Increase Position
      if FPosition < FMax then
        Position := FPosition + 1;

    vk_Down:                                          // Decrease Position
      if FPosition > FMin then
        Position := FPosition - 1;
  end; { case }
end; {= TRkTrackBar.KeyDown =}


procedure TRkTrackBar.MouseDown( Button : TMouseButton;
                                 Shift : TShiftState; X, Y : Integer );
var
  PtX, PtY : Integer;
  Delta : Real;
begin
  inherited MouseDown( Button, Shift, X, Y );

  if not ( csDesigning in ComponentState ) then
    SetFocus;                    // Move focus to TrackBar only at runtime

  if ( Button = mbLeft ) and PtInRect( FThumbRct, Point( X, Y ) ) then
  begin
    // User pressed the left mouse button while on the thumb
    FSliding := True;
    SetCapture( Handle );
  end
  else if ( Button = mbLeft ) and PtInRect( FTrackRct, Point(X, Y) ) then
  begin
    // User pressed left mouse button inside the track on either side
    // of thumb. Determine which side of thumb user clicked, and then
    // update position.

    if FOrientation = toVertical then
    begin
      Delta := (Height - FThumbWidth - 2 * FBorderWidth) / (FMax - FMin);
      PtY := Trunc( Delta * ((FMax - FPosition) - FMin) ) + FBorderWidth;
      if Y < PtY then
        Position := FPosition + FPageSize
      else
        Position := FPosition - FPageSize;
    end
    else
    begin
      Delta := ( Width - FThumbWidth - 2 * FBorderWidth ) / (FMax - FMin);
      PtX := Trunc( Delta * ( FPosition - FMin ) ) + FBorderWidth;
      if X < PtX then
        Position := FPosition - FPageSize
      else
        Position := FPosition + FPageSize;
    end;
  end;
end; {= TRkTrackBar.MouseDown =}


function TRkTrackBar.CursorPosition : TPoint;
begin
  GetCursorPos( Result );
  Result := ScreenToClient( Result );
end;


{=========================================================================
  TRkTrackBar.WMSetCursor

  If the mouse is over the thumb, then change cursor to the custom thumb
  cursor that resides in the RzTrkBar.res resource file.
=========================================================================}

procedure TRkTrackBar.WMSetCursor( var Msg : TWMSetCursor );
begin
  if PtInRect( FThumbRct, CursorPosition ) then
    SetCursor( FThumbCursor )
  else
    inherited;
end;


procedure TRkTrackBar.MouseMove( Shift : TShiftState; X, Y : Integer );
var
  P, W, H : Integer;
begin
  inherited MouseMove( Shift, X, Y );

  // If in Sliding state, then move the thumb to the closest tick mark.
  if FSliding then
  begin
    if FOrientation = toVertical then
    begin
      H := Height - FHalfWidth;
      P := Round( ( ( H - Y ) / H ) * ( FMax - FMin ) + FMin );
    end
    else
    begin
      W := Width - FHalfWidth;
      P := Round( ( ( X - FHalfWidth ) / W ) * ( FMax - FMin ) + FMin  );
    end;

    if P > FMax then
      P := FMax;
    if P < FMin then
      P := FMin;
    Position := P;
  end;
end; {= TRkTrackBar.MouseMove =}


procedure TRkTrackBar.MouseUp( Button : TMouseButton; Shift : TShiftState;
                               X, Y : Integer );
begin
  inherited MouseUp( Button, Shift, X, Y );

  if FSliding and ( Button = mbLeft ) then
  begin
    FSliding := False;
    ReleaseCapture;
  end;
end;


procedure TRkTrackBar.WMGetDlgCode( var Msg : TWMGetDlgCode );
begin
  inherited;
  Msg.Result := dlgc_WantArrows;     // So TrackBar can process arrow keys
end;


procedure TRkTrackBar.WMSize( var Msg : TWMSize );
begin
  inherited;
  if Height > Width then
    Orientation := toVertical
  else
    Orientation := toHorizontal;
end;


procedure TRkTrackBar.CMEnabledChanged( var Msg : TMessage );
begin
  inherited;
  Invalidate;
end;


{=========================================================================
  TRkTrackBar.CMDesignHitTest

  Handling this component message allows the TrackBar's thumb to be
  changed at design-time using the left mouse button.  If the mouse is
  positioned over the thumb or the thumb is already being dragged, then
  set the Msg.Result value to 1. This instructs Delphi to allow mouse
  events to "get to" the TrackBar component.
=========================================================================}

procedure TRkTrackBar.CMDesignHitTest( var Msg : TCMDesignHitTest );
begin
  if FSliding or PtInRect( FThumbRct, CursorPosition ) then
    Msg.Result := 1
  else
    Msg.Result := 0;
end;


end.
